function sol = fcmin(fcn,x0,H0,grad,options,varargin)
% sol = fcmin(fcn,x0,H0,grad,options,varargin)
% fcn:   string naming the objective function to be minimized
% x0:    initial value of the parameter vector
% H0:    initial value for the inverse Hessian.  Must be positive definite.
% grad:  Either a string naming a function that calculates the gradient, or the null matrix.
%        If it's null, the program calculates a numerical gradient.  In this case fcn must
%        be written so that it can take a matrix argument and produce a row vector of values.
%
% crit:  Convergence criterion.  Iteration will cease when it proves impossible to improve the
%        function value by more than crit.
% nit:   Maximum number of iterations.
% varargin: A list of optional length of additional parameters that get handed off to fcn each
%        time it is called.
%        Note that if the program ends abnormally, it is possible to retrieve the current x,
%        f, and H from the files g1.mat and H.mat that are written at each iteration and at each
%        hessian update, respectively.  (When the routine hits certain kinds of difficulty, it
%        write g2.mat and g3.mat as well.  If all were written at about the same time, any of them
%        may be a decent starting point.  One can also start from the one with best function value.)
%
x0      = x0(:);
nx      = size(x0,1);
if ~isempty(options)
    crit    = options.crit;
    nit     = options.nit;
    Verbose = options.verb;
    House   = options.house;
    Pr_     = options.display;
else
    crit    = 1e-5;
    nit     = 1000;
    Verbose = 0;
    House   = 1;
    Pr_     = 1;
end

NumGrad = isempty(grad);
done    = 0;
itct    = 0;
fcount  = 0;
snit    = 100;
f0      = feval(fcn,x0,varargin{:});
retcode3= 101;
x       = x0;
f       = f0;
H       = H0;
cliff   = 0;

if f0 > 1e50
    error('Bad initial parameter.')
end

if NumGrad
    if length(grad)==0
        [g,badg]= jacobian(fcn,x0, varargin{:});
    else
        badg    = any(find(grad==0));
        g       = grad;
    end
else
    [g,badg]    = feval(grad,x0,varargin{:});
end

while ~done
    g1=[];
    g2=[];
    g3=[];
    if Pr_
        disp('------------------------------------------------------------------------------------------')
        fprintf('f at the beginning of new iteration, %20.10f\n\n',f)
        disp('Vector of variables (x):')
        fprintf('%15.8g %15.8g %15.8g %15.8g %15.8g\n',x);
        disp(' ')
    end
    %-------------------------
    itct                = itct+1;
    [f1,x1,fc,retcode1] = fcminit(fcn,x,f,g,badg,H,Pr_,varargin{:});
    fcount              = fcount+fc;
    if retcode1 ~= 1
        if retcode1==2 | retcode1==4
            wall1   = 1;
            badg1   = 1;
        else
            if NumGrad
                [g1,badg1]  = jacobian(fcn, x1,varargin{:});
            else
                [g1,badg1]  = feval(grad,x1,varargin{:});
            end
            wall1   = badg1;
            if House
                save g1.mat g1 x1 f1 varargin;
            end
        end
        if wall1
            Hcliff  = H+diag(diag(H).*rand(nx,1));
            if Pr_
                disp('Cliff.  Perturbing search direction.')
                disp(' ')
            end
            [f2,x2,fc,retcode2] = fcminit(fcn,x,f,g,badg,Hcliff,Pr_,varargin{:});
            fcount = fcount+fc; % put by Jinill
            if  f2 < f
                if retcode2==2 | retcode2==4
                    wall2   = 1;
                    badg2   = 1;
                else
                    if NumGrad
                        [g2,badg2]  = jacobian(fcn, x2,varargin{:});
                    else
                        [g2,badg2]  = feval(grad,x2,varargin{:});
                    end
                    wall2   = badg2;
                    if House
                        save g2.mat g2 x2 f2 varargin;
                    end
                end
                if wall2
                    if Pr_
                        disp('Cliff again.  Try traversing')
                        disp(' ')
                    end
                    if norm(x2-x1) < 1e-13
                        f3      = f;
                        x3      = x;
                        badg3   = 1;
                        retcode3= 101;
                    else
                        gcliff=((f2-f1)/((norm(x2-x1))^2))*(x2-x1);
                        if(size(x0,2)>1)
                            gcliff  = gcliff'
                        end
                        [f3,x3,fc,retcode3] = fcminit(fcn,x,f,gcliff,0,eye(nx),Pr_,varargin{:});
                        fcount              = fcount+fc;
                        if retcode3==2 | retcode3==4
                            wall3   = 1;
                            badg3   = 1;
                        else
                            if NumGrad
                                [g3,badg3]  = jacobian(fcn, x3,varargin{:});
                            else
                                [g3,badg3]  = feval(grad,x3,varargin{:});
                            end
                            wall3   = badg3;
                            if House
                                save g3.mat g3 x3 f3 varargin;
                            end
                        end
                    end
                else
                    f3      = f;
                    x3      = x;
                    badg3   = 1;
                    retcode3= 101;
                end
            else
                f3      = f;
                x3      = x;
                badg3   = 1;
                retcode3= 101;
            end
        else
            f2      = f;
            f3      = f;
            badg2   = 1;
            badg3   = 1;
            retcode2= 101;
            retcode3= 101;
        end
    else
        f2      = f;
        f3      = f;
        f1      = f;
        retcode2= retcode1;
        retcode3= retcode1;
    end
    if f3<f & badg3==0
        ih      = 3;
        fh      = f3;
        xh      = x3;
        gh      = g3;
        badgh   = badg3;
        retcodeh=retcode3;
    elseif f2<f & badg2==0
        ih      = 2;
        fh      = f2;
        xh      = x2;
        gh      = g2;
        badgh   = badg2;
        retcodeh= retcode2;
    elseif f1<f & badg1==0
        ih      = 1;
        fh      = f1;
        xh      = x1;
        gh      = g1;
        badgh   = badg1;
        retcodeh= retcode1;
    else
        [fh,ih] = min([f1,f2,f3]);
        switch ih
            case 1
                xh  = x1;
            case 2
                xh  = x2;
            case 3
                xh  = x3;
        end
        retcodei= [retcode1,retcode2,retcode3];
        retcodeh= retcodei(ih);
        if exist('gh')
            nogh= isempty(gh);
        else
            nogh= 1;
        end
        if nogh
            if NumGrad
                [gh,badgh]  = jacobian(fcn,xh,varargin{:});
            else
                [gh,badgh]  = feval(grad, xh,varargin{:});
            end
        end
        badgh   = 1;
    end
    stuck   = (abs(fh-f) < crit);
    if (~badg)&(~badgh)&(~stuck)
        H   = fcbfgs(H,gh-g,xh-x);
    end
    if Pr_
        fprintf('Improvement at iteration %d: %16.8f\n',itct,f-fh)
    end
    if Verbose
        if itct > nit
            disp('iteration count termination')
            done = 1;
        elseif stuck
            disp('improvement < crit termination')
            done = 1;
        end
        rc=retcodeh;
        if rc == 1
            disp('zero gradient')
        elseif rc == 6
            disp('smallest step still improving too slow, reversed gradient')
        elseif rc == 5
            disp('largest step still improving too fast')
        elseif (rc == 4) | (rc==2)
            disp('back and forth on step length never finished')
        elseif rc == 3
            disp('smallest step still improving too slow')
        elseif rc == 7
            disp('warning: possible inaccuracy in H matrix')
        end
    end
    f   = fh;
    x   = xh;
    g   = gh;
    badg= badgh;
end
sol.x       = xh;
sol.f       = fh;
sol.jac     = gh;
sol.ihess   = H;
sol.term    = retcodeh;
sol.fcount  = fcount;
sol.iter    = itct;
return
%
% function used by the method
%
function [fhat,xhat,fcount,retcode] = fcminit(fcn,x0,f0,g0,badg,H0,Pr_,varargin)
% [fhat,xhat,fcount,retcode] = fcminit(fcn,x0,f0,g0,badg,H0,...
%                                       P1,P2,P3,P4,P5,P6,P7,P8)
% retcodes: 0, normal step.  5, largest step still improves too fast.
% 4,2 back and forth adjustment of stepsize didn't finish.  3, smallest
% stepsize still improves too slow.  6, no improvement found.  1, zero
% gradient.
ANGLE   = 0.005;
THETA   = 0.3; %(0<THETA<.5) THETA near .5 makes long line searches, possibly fewer iterations.
FCHANGE = 1000;
MINLAMB = 1e-9;
MINDFAC = 0.01;
fcount  = 0;
lambda  = 1;
xhat    = x0;
f       = f0;
fhat    = f0;
g       = g0;
gnorm   = norm(g);

if (gnorm < 1.e-12) & ~badg
    retcode = 1;
    dxnorm  = 0;
else
    dx      =-H0*g;
    dxnorm  = norm(dx);
    if dxnorm > 1e12
        if Pr_
            disp('Near-singular H problem.')
        end
        dx  = dx*FCHANGE/dxnorm;
    end
    dfhat   = dx'*g0;
    if ~badg
        % test for alignment of dx with gradient and fix if necessary
        a   = -dfhat/(gnorm*dxnorm);
        if a<ANGLE
            dx      = dx - (ANGLE*dxnorm/gnorm+dfhat/(gnorm*gnorm))*g;
            dfhat   = dx'*g;
            dxnorm  = norm(dx);
            %             fprintf('Correct for low angle: %g\n',a))
            %             disp(' ')
        end
    end
    if Pr_
        fprintf('Predicted improvement: %16.8f\t',-dfhat/2)
    end
    done        = 0;
    factor      = 3;
    shrink      = 1;
    lambdaMin   = 0;
    lambdaMax   = inf;
    lambdaPeak  = 0;
    fPeak       = f0;
    lambdahat   = 0;
    while ~done
        dxtest  = x0+dx*lambda;
        f       = feval(fcn,dxtest,varargin{:});
        if f<fhat
            fhat        = f;
            xhat        = dxtest;
            lambdahat   = lambda;
        end
        fcount      = fcount+1;
        shrinkSignal= (~badg & (f0-f < max([-THETA*dfhat*lambda 0]))) | (badg & (f0-f) < 0) ;
        growSignal  = ~badg & ( (lambda > 0)  &  (f0-f > -(1-THETA)*dfhat*lambda) );
        if  shrinkSignal  &   ( (lambda>lambdaPeak) | (lambda<0) )
            if (lambda>0) & ((~shrink) | (lambda/factor <= lambdaPeak))
                shrink  = 1;
                factor  = factor^.6;
                while lambda/factor <= lambdaPeak
                    factor  = factor^.6;
                end
                if abs(factor-1)<MINDFAC
                    if abs(lambda)<4
                        retcode = 2;
                    else
                        retcode = 7;
                    end
                    done    = 1;
                end
            end
            if (lambda<lambdaMax) & (lambda>lambdaPeak)
                lambdaMax   = lambda;
            end
            lambda  = lambda/factor;
            if abs(lambda) < MINLAMB
                if (lambda > 0) & (f0 <= fhat)
                    % try going against gradient, which may be inaccurate
                    lambda  =-lambda*factor^6;
                else
                    if lambda < 0
                        retcode = 6;
                    else
                        retcode = 3;
                    end
                    done    = 1;
                end
            end
        elseif  (growSignal & lambda>0) |  (shrinkSignal & ((lambda <= lambdaPeak) & (lambda>0)))
            if shrink
                shrink  = 0;
                factor  = factor^.6;
                %if ( abs(lambda)*(factor-1)*dxnorm< MINDX ) | ( abs(lambda)*(factor-1)< MINLAMB)
                if abs(factor-1)<MINDFAC
                    if abs(lambda)<4
                        retcode = 4;
                    else
                        retcode = 7;
                    end
                    done    = 1;
                end
            end
            if ( f<fPeak ) & (lambda>0)
                fPeak       = f;
                lambdaPeak  = lambda;
                if lambdaMax<=lambdaPeak
                    lambdaMax   = lambdaPeak*factor*factor;
                end
            end
            lambda  = lambda*factor;
            if abs(lambda) > 1e20;
                retcode = 5;
                done    = 1;
            end
        else
            done    = 1;
            if factor < 1.2
                retcode = 7;
            else
                retcode = 0;
            end
        end
    end
end
% fprintf('Norm of dx: %10.5g\n', dxnorm))
return

function H = fcbfgs(H0,dg,dx)
% H = fcbfgs(H0,dg,dx)
% dg is previous change in gradient; dx is previous change in x;
% 6/8/93 version that updates inverse hessian instead of hessian
% itself.
% Copyright by Christopher Sims 1996.  This material may be freely
% reproduced and modified.
Hdg     = H0*dg;
dgdx    = dg'*dx;
if (abs(dgdx) >1e-12)
    H   = H0 + (1+(dg'*Hdg)/dgdx)*(dx*dx')/dgdx - (dx*Hdg'+Hdg*dx')/dgdx;
else
    disp('bfgs update failed.')
    disp(['|dg| = ' num2str(sqrt(dg'*dg)) '|dx| = ' num2str(sqrt(dx'*dx))]);
    disp(['dg''*dx = ' num2str(dgdx)])
    disp(['|H*dg| = ' num2str(Hdg'*Hdg)])
    H   = H0;
end
return

function [g, badg] = jacobian(fcn,x,varargin)
x       = x(:);
n       = size(x,1);
% delta   = 1e-6;
tvec    = diag(.00001*max(abs(x),1e-8*ones(size(x))));
g       = zeros(n,1);
f0      = feval(fcn,x,varargin{:});
badg    = 0;
for i=1:n
    scale= 1; % originally 1
    tvecv= tvec(:,i);
    g0   = (feval(fcn,x+scale*tvecv,varargin{:})-f0)/(scale*tvec(i,i));
    if abs(g0)< 1e15
        g(i)  = g0;
    else
        disp('bad gradient ------------------------') % Jinill Kim
        g(i)  = 0;
        badg  = 1;
    end
end
return