function [x,f]=kdens(data,kern,nbx,h,range);

% KDENS       KDE plot for 1D data using FFTs and the Normal kernel
%
%             Example: load kddata
%                      kdfft1(suicide,'kn',128,40);
%
%             kdens(data,K,INC,H,range) plots the Kernel Density Estimate with Normal kernel.  
%             data : The data is stored in the rows of data.
%             kern : The kernel function, one of 'KNORM', 'KEPAN', etc..
%             nbx  : The number of increments to use in plotting the density function.  
%                    THIS MUST BE data POWER OF 2, for example 32, 64 and 128 give good results.  
%             h    : Window width.  If omitted, a simplistic 'optimal' value is used.
%             range: Optional, range of the data
%
%             Christian C. Beardah 1994 modified F. Collard, 2002

if ~isempty(kern);
    kern    = lower(kern);
else
    kern    = 'knorm';
end;

data= data(:);
n   = size(data,1);     % n=number of data points,
P   = 2*nbx;
if kern=='knorm',
    r = 4;
elseif kern=='klogi' | kern=='klapl',
    r   = 6;
else,
    r = 1;
end;


if h==0,
    switch kern;
    case 'knorm', % Normal kernel
        a=1.0592;
    case 'kepan', % Epanechnikov kernel
        a=2.34;
    case 'kbiwe', % Bi-weight kernel
        a=2.7779;
    case 'ktria', % Triangular
        a=2.5760;
    case 'krect', % Rectangular
        a=1.8431;
    case 'klapl', % Laplace
        a=0.7836;
    case 'ktriw', % Triweight
        a=3.1545;
    case 'klogi', % Logistic
        a=0.5921;
    end;
    
    if siqr(data)>0,
        s=min([std(data) siqr(data)/1.34]);
    else,
        s=std(data);
    end;
    
    h=a*s*n^(-1/5);
end;

if nargin<5;
    xmin    = min(data);      % Find the minimum value of data.
    xmax    = max(data);      % Find the maximum value of data.
    xrange  = xmax-xmin; % Find the range of data.
    
    % xa holds the x 'axis' vector, defining a grid of x values where 
    % the k.d. function will be evaluated and plotted.
    
    ax      = xmin-xrange/4;
    bx      = xmax+xrange/4;
else
    ax      = range(1);
    bx      = range(2);
end

x   = linspace(ax,bx,nbx); 

c   = zeros(nbx,1);
dx  = (bx-ax)/(nbx-1);
binx= floor((data-ax)/dx)+1;
% Obtain the grid counts.
for i=1:n, % Loop over data points in x direction.
    c(binx(i))  = c(binx(i))+(x(binx(i)+1)-data(i))/dx;
    c(binx(i)+1)= c(binx(i)+1)+(data(i)-x(binx(i)))/dx;  
end;

% Obtain the kernel weights.

L                   = min(floor(r*h*(nbx-1)/(bx-ax)),nbx-1);
kw                  = zeros(1,2*nbx);
temp                = feval(kern,(bx-ax)*[0:L]/((nbx-1)*h))/(n*h);
kw(nbx+1:nbx+L+1)   = temp;
kw(nbx-L+1:nbx)     = temp([L+1:-1:2]);
kw                  = fftshift(kw)';
%
% Perform the convolution.
%
f                   = real(ifft(fft(c,P).*fft(kw)));
f                   = f(1:nbx).*(f(1:nbx)>0);
